using Microsoft.Extensions.Logging;
using SoftwareInstallationBusinessLogic.MailWorker;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.BusinessLogicsContracts;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SoftwareInstallationBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractMailWorker _mailWorker;
        private readonly IClientLogic _clientLogic;
        static readonly object locker = new object();

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }

            model.Status = OrderStatus.Принят;

            if (_orderStorage.Insert(model) == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
        {
            var viewModel = _orderStorage.GetElement(new() { Id = model.Id });
            if (viewModel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (viewModel.Status + 1 != newStatus)
            {
                _logger.LogWarning("Обновление статуса на " + newStatus.ToString() + " вызвало ошибку. Статус заказа некорректен.");
                return false;
            }

            model.Status = newStatus;

            if (model.Status == OrderStatus.Готов)
                model.DateImplement = DateTime.Now;

            if (viewModel.ImplementerId.HasValue)
                model.ImplementerId = viewModel.ImplementerId;

            model.Sum = viewModel.Sum;
            model.Count = viewModel.Count;
            model.PackageId = viewModel.PackageId;


            if (_orderStorage.Update(model) == null)
            {
                model.Status--;
                _logger.LogWarning("Неудачная операция обновления статуса");
                return false;
            }

            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (locker)
            {
                return StatusUpdate(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Order. OrderId:{Id}", model?.Id);

            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (model.PackageId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор изделия", nameof(model.PackageId));
            }

            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }

            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }

            _logger.LogInformation("Order. OrderId:{Id}.Sum:{Sum}. PackageId: {PackageId}", model.Id, model.Sum, model.PackageId);
        }
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);

            var element = _orderStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);

            return element;
        }
    }

}
